
/* File:        org.molgenis.omx/model/StudyDataRequest.java
 * Copyright:   GBIC 2000-2013, all rights reserved
 * Date:        August 21, 2013
 * 
 * generator:   org.molgenis.generators.db.EntityImporterGen 4.0.0-testing
 *
 * 
 * THIS FILE HAS BEEN GENERATED, PLEASE DO NOT EDIT!
 */

package org.molgenis.omx.filter.db;

import java.io.IOException;
import java.util.ArrayList;
import java.util.LinkedHashSet;
import java.util.List;
import java.util.Map;
import java.util.TreeMap;
import java.util.concurrent.atomic.AtomicInteger;

import org.apache.commons.lang3.StringUtils;
import com.google.common.collect.Sets;
import org.apache.log4j.Logger;
import org.molgenis.framework.db.Database;
import org.molgenis.framework.db.DatabaseException;
import org.molgenis.framework.db.Database.DatabaseAction;
import org.molgenis.framework.db.EntityImporter;
import org.molgenis.io.processor.LowerCaseProcessor;
import org.molgenis.io.TupleReader;
import org.molgenis.util.tuple.Tuple;

import org.molgenis.omx.observ.ObservableFeature;
import org.molgenis.omx.auth.MolgenisUser;
import org.molgenis.omx.filter.StudyDataRequest;


/**
 * Reads StudyDataRequest from a delimited (csv) file, resolving xrefs to ids where needed, that is the tricky bit ;-)
 */
public class StudyDataRequestEntityImporter implements EntityImporter
{
	private static final Logger logger = Logger.getLogger(StudyDataRequestEntityImporter.class);
	
	private static int BATCH_SIZE = 10000;
	//foreign key map for xref 'features' (maps observableFeature.Identifier -> observableFeature.id)			
	final Map<String,Integer> featuresKeymap = new TreeMap<String,Integer>();	
	//foreign key map for xref 'molgenisUser' (maps molgenisUser.Name -> molgenisUser.id)			
	final Map<String,Integer> molgenisUserKeymap = new TreeMap<String,Integer>();	
			
	/**
	 * Imports StudyDataRequest from tab/comma delimited File
	 * @param db database to import into
	 * @param reader csv reader to load data from
	 * @param defaults to set default values for each row
	 * @param dbAction indicating wether to add,update,remove etc
	 * @param missingValues indicating what value in the csv is treated as 'null' (e.g. "" or "NA")
	 * @return number of elements imported
	 */
	@Override
	public int importEntity(TupleReader reader, Database db, DatabaseAction dbAction) throws IOException, DatabaseException 
	{
		// normalize column headers
		reader.addCellProcessor(new LowerCaseProcessor(true, false));
		//wrapper to count
		final AtomicInteger total = new AtomicInteger(0);
	try {
		//cache for entities of which xrefs couldn't be resolved (e.g. if there is a self-refence)
		//these entities can be updated with their xrefs in a second round when all entities are in the database
		List<StudyDataRequest> studyDataRequestsMissingRefs = new ArrayList<StudyDataRequest>();
	
		//cache for objects to be imported from file (in batch)
		final List<StudyDataRequest> studyDataRequestList = new ArrayList<StudyDataRequest>(BATCH_SIZE); // FIXME
		
		for(Tuple tuple : reader)
		{
			// skip empty rows
			if (!hasValues(tuple)) continue;
			
			//parse object, setting defaults and values from file
			StudyDataRequest object = new StudyDataRequest();
			object.set(tuple, false);				
			studyDataRequestList.add(object);		
			
			//add to db when batch size is reached
			if(studyDataRequestList.size() == BATCH_SIZE)
			{
				//resolve foreign keys and copy those entities that could not be resolved to the missingRefs list
				studyDataRequestsMissingRefs.addAll(resolveForeignKeys(db, studyDataRequestList));
				studyDataRequestList.removeAll(studyDataRequestsMissingRefs);
				
				//update objects in the database using xref_label defined secondary key(s) 'Identifier' defined in xref_label
				db.update(studyDataRequestList,dbAction, "Identifier");
				
				//clear for next batch						
				studyDataRequestList.clear();		
				
				//keep count
				total.set(total.get() + BATCH_SIZE);
				
				db.getEntityManager().flush();
				db.getEntityManager().clear();		
			}
		}
			
		//add remaining elements to the database
		if(!studyDataRequestList.isEmpty())
		{
			total.set(total.get() + studyDataRequestList.size());
			
			//resolve foreign keys, again keeping track of those entities that could not be solved
			studyDataRequestsMissingRefs.addAll(resolveForeignKeys(db, studyDataRequestList));
			studyDataRequestList.removeAll(studyDataRequestsMissingRefs);
			
			//update objects in the database using xref_label defined secondary key(s) 'Identifier' defined in xref_label
			db.update(studyDataRequestList,dbAction, "Identifier");
		}
		
		//Try to resolve FK's for entities until all are resolved or we have more then 100 iterations
		List<StudyDataRequest> studyDataRequests = new ArrayList<StudyDataRequest>(studyDataRequestsMissingRefs);

		int iterationCount = 0;

		do
		{
			studyDataRequestsMissingRefs = resolveForeignKeys(db, studyDataRequestsMissingRefs);
			
			LinkedHashSet<StudyDataRequest> differenceSet = new LinkedHashSet<StudyDataRequest>();
			Sets.symmetricDifference(new LinkedHashSet<StudyDataRequest>(studyDataRequests), new LinkedHashSet<StudyDataRequest>(studyDataRequestsMissingRefs)).copyInto(differenceSet);
			List<StudyDataRequest> resolvablestudyDataRequests = new ArrayList<StudyDataRequest>(differenceSet);
			
			studyDataRequests.removeAll(resolvablestudyDataRequests);
			
			db.update(resolvablestudyDataRequests,dbAction, "Identifier");

			if (iterationCount++ > 100)
			{
			String identifier = "";
			String name = "";
				for(StudyDataRequest blaat : studyDataRequestsMissingRefs){
					identifier = blaat.getValues().get("Identifier").toString();
					name = blaat.getValues().get("Name").toString();
				}
				throw new Exception(
						"Import of 'studyDataRequest' entity failed:"
								+ "This is probably caused by a(n) 'studyDataRequest' that has a reference but that does not exist."
								+"(identifier:"+identifier+", name:"+name+")");		
			}
		}
		while (studyDataRequestsMissingRefs.size() > 0);

		logger.info("imported " + total.get() + " studyDataRequest from CSV");

		} catch(Exception e) {throw new IOException(e);}
		return total.get();
	}	
	
	private boolean hasValues(Tuple tuple)
	{
		for (String colName : tuple.getColNames())
		{
			if (tuple.get(colName) != null) return true;
		}
		return false;
	}
	
	/**
	 * This method tries to resolve foreign keys (i.e. xref_field) based on the secondary key/key (i.e. xref_labels).
	 *
	 * @param db database
	 * @param studyDataRequestList 
	 * @return the entities for which foreign keys cannot be resolved
	 */
	private List<StudyDataRequest> resolveForeignKeys(Database db, List<StudyDataRequest> studyDataRequestList) throws Exception
	{
		//keep a list of StudyDataRequest instances that miss a reference which might be resolvable later
		List<StudyDataRequest> studyDataRequestsMissingRefs = new ArrayList<StudyDataRequest>();
	
		//resolve xref 'features' from observableFeature.Identifier -> observableFeature.id
		for(StudyDataRequest o: studyDataRequestList) for(String xref_label: o.getFeatures_Identifier())
		{
			if(xref_label != null) 
				featuresKeymap.put(xref_label.trim(), null);
		}
		
		if(featuresKeymap.size() > 0) 
		{
			List<ObservableFeature> featuresList = db.query(ObservableFeature.class).in("Identifier",new ArrayList<Object>(featuresKeymap.keySet())).find();
			for(ObservableFeature xref :  featuresList)
			{
				featuresKeymap.put(xref.getIdentifier(), xref.getId());
			}
		}
		//resolve xref 'molgenisUser' from molgenisUser.Name -> molgenisUser.id
		for(StudyDataRequest o: studyDataRequestList) 
		{
			if(o.getMolgenisUser_Name() != null) 
				molgenisUserKeymap.put(o.getMolgenisUser_Name(), null);
		}
		
		if(molgenisUserKeymap.size() > 0) 
		{
			List<MolgenisUser> molgenisUserList = db.query(MolgenisUser.class).in("Name",new ArrayList<Object>(molgenisUserKeymap.keySet())).find();
			for(MolgenisUser xref :  molgenisUserList)
			{
				molgenisUserKeymap.put(xref.getName(), xref.getId());
			}
		}
		//update objects with foreign key values
		for(StudyDataRequest o:  studyDataRequestList)
		{
			while(true){
				//update mref Features
				if(o.getFeatures_Identifier() != null) 
				{
					List<Integer> mrefs = new ArrayList<Integer>();
					boolean breakToNextStudyDataRequest = false;

					int listSize = 0;
					if(o.getFeatures_Identifier() != null) listSize = Math.max(o.getFeatures_Identifier().size(), listSize);
					for(int i = 0; i < listSize; i++)
					{
						String key = o.getFeatures_Identifier().get(i);
							key = key.trim();				
						if(featuresKeymap.get(key) == null){
							logger.error("Import of 'StudyDataRequest' objects failed: "+o);
							throw new Exception("Import of 'StudyDataRequest' objects failed:" 
							+"cannot find features_Identifier='"+(o.getFeatures_Identifier() != null && i < o.getFeatures_Identifier().size() ? o.getFeatures_Identifier().get(i) : "null")+"'");
						}
						mrefs.add(featuresKeymap.get(key));
					}
					if(breakToNextStudyDataRequest){
						break;
					}
					o.setFeatures_Id(mrefs);
				}
				//update xref MolgenisUser
				if(o.getMolgenisUser_Name() != null) 
				{
					String key = o.getMolgenisUser_Name();
					if(molgenisUserKeymap.get(key) == null)
					{
						throw new Exception("Import of 'StudyDataRequest' objects failed: cannot find MolgenisUser for molgenisUser_Name='"+o.getMolgenisUser_Name()+"'");
					}
					o.setMolgenisUser_Id(molgenisUserKeymap.get(key));
				}
				break;
			}
		}
		
		featuresKeymap.clear();
		molgenisUserKeymap.clear();
		
		return studyDataRequestsMissingRefs;
	}
}

